;
; Checks the documentation for various errors

#Region Includes
#include "Array.au3"
#include "include\OutputLib.au3"
#include "include\SettingsLib.au3"
#EndRegion Includes

#Region Global Variables
; The name of the project.
Global Const $g_sProject = "Check Include Headers"
Global Const $g_sProjectDir = "install\include"

#EndRegion Global Variables

#Region Main body of code
Global $g_nExitCode = Main($g_sProject)
Exit $g_nExitCode
#EndRegion Main body of code

#Region Main()
; ===================================================================
; Main()
;
; The main program body.
; Parameters:
;	None.
; Returns:
;	None.
; ===================================================================
Func Main($sProject)
	Local $nReturn = 0

	; Create the output window and initial message.
	_OutputWindowCreate()
	_OutputProgressWrite("==== Output for " & StringTrimRight(@ScriptName, StringLen(".exe")) & " (" & $sProject & ") ====" & @CRLF)
	_OutputProgressWrite("Checking... ")

	; Set the build directory based on the rules and the INI file value.
	_SettingBuildDir()

	; Check Include files header.
	$nReturn = Check_includesHeader($g_sProjectDir)
	ToolTip("")

	; Check the return value.
	If $nReturn Then
		_OutputProgressWrite("failed (" & $nReturn & ")." & @CRLF)
	Else
		_OutputProgressWrite("complete." & @CRLF)
	EndIf

	; Write closing message and wait for close (if applicable).
	_OutputProgressWrite("Finished." & @CRLF & @CRLF) ; Two CRLF's in case of chained output.
	_OutputWaitClosed($nReturn)

	; Return the value.
	Return $nReturn
EndFunc   ;==>Main
#EndRegion Main()

#Region Primary Functions

#Region Check_includesHeader()

Func Check_includesHeader($sIncludeDir)
	Local $nOK = 0, $nReturn = 0
	Local $n, $sFile, $sIncludeFile, $aCurrentFunctions, $sCurrentFunction, $sFunction, $aFunctions
	Local $hSearch = FileFindFirstFile($sIncludeDir & "\*.au3")
	While 1
		$sFile = FileFindNextFile($hSearch)
		If @error Then ExitLoop

		$sIncludeFile = FileRead($sIncludeDir & "\" & $sFile)
		$aCurrentFunctions = GetCurrentFunctions($sIncludeFile)
		ToolTip($sFile, 200, 200)
		If IsArray($aCurrentFunctions) Then
			$nReturn += 1
			$nOK = 0
			$n = 0
			For $sCurrentFunction In $aCurrentFunctions
				If StringLeft($sCurrentFunction, 1) = "_" Then
					If StringInStr($sIncludeFile, "Func " & $sCurrentFunction) = 0 Then
						$n += 1
						_OutputProgressWrite("Function " & $sCurrentFunction & " not found in " & $sFile & @CRLF)
					EndIf
				Else
						_OutputProgressWrite("??? " & $sCurrentFunction & " not found in " & $sFile & @CRLF)
				EndIf
			Next
			If $n = 0 Then
				$nOK = 1
				$nReturn -= 1 ; no error
			EndIf

			$aFunctions = GetFunctions($sIncludeFile)
			For $sFunction In $aFunctions
				$n = 0
				For $sCurrentFunction In $aCurrentFunctions
					If StringLeft($sCurrentFunction, 1) = "_" Then
						If $sFunction = $sCurrentFunction Then
							$n = 1
							ExitLoop
						EndIf
					EndIf
				Next
				If $n = 0 Then
					If $nOK Then $nReturn += 1
					_OutputProgressWrite($sFunction & "() not found in CURRENT section of " & $sFile & @CRLF)
				EndIf
			Next
		Else
			If StringInStr($sFile, "Constants") Then ContinueLoop
			If StringInStr($sFile, "SQLite.dll") Then ContinueLoop
			If StringInStr($sFile, "UDFGlobalID.au3") Then ContinueLoop
	 	 	If StringInStr($sFile, "WinAPI.au3") Then ContinueLoop
			If StringInStr($sFile, "WinAPIEx.au3") Then ContinueLoop
			If StringInStr($sFile, "Internals.au3") Then ContinueLoop

			_OutputBuildWrite("No CURRENT section in " & $sFile & @CRLF)
		EndIf
	WEnd

	; Close the search handle
	FileClose($hSearch)

	Return $nReturn

EndFunc   ;==>Check_includesHeader

#EndRegion Check_includesHeader()

#Region GetCurrentFunctions()

Func GetCurrentFunctions($sIncludeFile)
	Local $aFiles
	Local $j, $i = StringInStr($sIncludeFile, "; #CURRENT# =")
	If $i Then
		$i = StringInStr($sIncludeFile, ";", 0, 2, $i)
		$j = StringInStr($sIncludeFile, "; ===", 0, 1, $i)
		$aFiles = StringRegExp(StringMid($sIncludeFile, $i, $j - $i), "; (.*)\r\n", $STR_REGEXPARRAYGLOBALMATCH)

		DeleteNonFunctions($aFiles)

		Local $aNoDoc = GetNoDocFunctions($sIncludeFile)
		If IsArray($aNoDoc) Then _ArrayAdd($aFiles, $aNoDoc)
	EndIf

	Return $aFiles
EndFunc   ;==>GetCurrentFunctions

#EndRegion GetCurrentFunctions()

#Region GetInternalsFunctions()

Func GetInternalsFunctions($sIncludeFile)
	Local $aFiles
	Local $j, $i = StringInStr($sIncludeFile, "; #INTERNAL_USE_ONLY# =")
	If $i Then
		$i = StringInStr($sIncludeFile, ";", 0, 2, $i)
		$j = StringInStr($sIncludeFile, "; ===", 0, 1, $i)
		$aFiles = StringRegExp(StringMid($sIncludeFile, $i, $j - $i), "; (.*)\r\n", $STR_REGEXPARRAYGLOBALMATCH)
	EndIf

	DeleteNonFunctions($aFiles)

	Return $aFiles
EndFunc   ;==>GetInternalsFunctions

#EndRegion GetInternalsFunctions()

#Region GetNoDocFunctions()

Func GetNoDocFunctions($sIncludeFile)
	Local $aFiles
	Local $j, $i = StringInStr($sIncludeFile, "; #NO_DOC_FUNCTION# =")
	If $i Then
		$i = StringInStr($sIncludeFile, ";", 0, 2, $i)
		$j = StringInStr($sIncludeFile, "; ===", 0, 1, $i)
		$aFiles = StringRegExp(StringMid($sIncludeFile, $i, $j - $i), "; (.*)\r\n", $STR_REGEXPARRAYGLOBALMATCH)
	EndIf

	DeleteNonFunctions($aFiles)

	Return $aFiles
EndFunc   ;==>GetNoDocFunctions

#EndRegion GetNoDocFunctions()

#Region DeleteNonFunctions()

Func DeleteNonFunctions(ByRef $aFiles)
	If Not IsArray($aFiles) Then Return
	Local $k = 0, $vRange = ""
	For $sFunctionName In $aFiles
		If StringLeft($sFunctionName, 1) <> "_" Then
			If $sFunctionName = $aFiles[$k] Then
				$vRange &= $k & ";"
			Else
;~ 				ErrorLogWrite($k, $fonc, $sFunctionName, $aFiles[$k] & " Diff for " & $k)
				_OutputProgressWrite("??? " & $sFunctionName & " not found in " & $aFiles[$k] & @CRLF)
			EndIf
		EndIf
		$k += 1
	Next
	If $vRange Then
		$vRange = StringTrimRight($vRange, 1)
		_ArrayDelete($aFiles, $vRange)
	EndIf
EndFunc   ;==>DeleteNonFunctions

#EndRegion DeleteNonFunctions()

#Region GetFunctions()

Func GetFunctions($sIncludeFile)
	Local $aFiles[500]
	Local $j = 1, $i, $n = 0, $sFunctionName

	While 1
		$i = StringInStr($sIncludeFile, @CRLF & "Func ", 0, 1, $j)
		If $i = 0 Then ExitLoop
		$i += 7
		$j = StringInStr($sIncludeFile, "(", 0, 1, $i)
		$sFunctionName = StringMid($sIncludeFile, $i, $j - $i)
		If StringLeft($sFunctionName, 2) = "__" Then ContinueLoop
		$aFiles[$n] = $sFunctionName
		$n += 1
	WEnd
	ReDim $aFiles[$n]

	Return $aFiles
EndFunc   ;==>GetFunctions

#EndRegion GetFunctions()

#EndRegion Primary Functions
