; ===================================================================
; Project: Help File Make
; Description: Creates valid HTML5 documentation based on the TXT docs.
; Author: guinness
; Remarks: HTML files use the file extension .htm.
; ===================================================================

#include <File.au3>
#include <StringConstants.au3>
#include <WinAPI.au3>

#include 'MiscLib.au3'
#include 'OutputLib.au3'
#include 'SciTELib.au3'
#include 'Help Check\HelpFileMisc.au3' ; HelpFileMisc by guinness.

#cs
	Major issues:
	None.

	Minor issues:
	Change: Improve the old that was stripped out for creating tables. The logic has been simplified, though I believe it can be improved further.
#ce

Opt('TrayIconDebug', 1)

OnAutoItExitRegister('_HelpGenerate_OnQuit') ; ### When run in standalone
_OutputWindowCreate() ; ### When run in standalone

; Get the previous working directory before it is overwritten with the location of udf_devs\docs. This is in local scope as it's only used in that scope.
Local Const $sPreviousWorkingDir = @WorkingDir

#Region Stripped from Help File Check.au3
; Set to the udf_devs\docs directory.
FileChangeDir('..\..')

Global Const $AUTOITDOCSOURCE = _GetFullPath('.\autoit\english\txt2htm'), _
		$AUTOITHTMLSOURCE = _GetFullPath('.\autoit\english\html'), _
		$AUTOITXDOCSOURCE = _GetFullPath('.\autoitx\english\txt2htm'), _
		$AUTOITXHTMLSOURCE = _GetFullPath('.\autoitx\english\html\com_interface'), _
		$INCLUDESOURCE = _GetFullPath('..\install\Include'), _
		$EXAMPLES = 'examples', _
		$LIBEXAMPLES = 'libExamples', _
		$TXTFUNCTIONS = 'txtfunctions', _
		$HTMLFUNCTIONS = 'functions', _
		$TXTLIBFUNCTIONS = 'txtlibfunctions', _
		$HTMLLIBFUNCTIONS = 'libfunctions', _
		$TXTKEYWORDS = 'txtKeywords', _
		$HTMLKEYWORDS = 'keywords', _
		$HTMLMACROS = 'macros', _
		$TXTMETHODS = 'txtMethods', _
		$HTMLMETHODS = 'methods', _
		$OPTIONAL = '[optional]'

; These have back/forward slashes appended.
Global Const $HELPGEN_RELATIVE_TXTKEYWORDS = '..\' & $TXTKEYWORDS & '\', _
		$HELPGEN_RELATIVE_HTMLKEYWORDS = '../' & $HTMLKEYWORDS & '/', _
		$HELPGEN_RELATIVE_TXTFUNCTIONS = '..\' & $TXTFUNCTIONS & '\', _
		$HELPGEN_RELATIVE_HTMLFUNCTIONS = '../' & $HTMLFUNCTIONS & '/', _
		$HELPGEN_RELATIVE_TXTLIBFUNCTIONS = '..\' & $TXTLIBFUNCTIONS & '\', _
		$HELPGEN_RELATIVE_HTMLLIBFUNCTIONS = '../' & $HTMLLIBFUNCTIONS & '/', _
		$HELPGEN_RELATIVE_HTMLMETHODS = '../' & $TXTMETHODS & '/', _
		$HELPGEN_RELATIVE_TXTMETHODS = '..\' & $HTMLMETHODS & '\'

; Slight speed increase when storing macros to constants.
Global Const $CRLF = @CRLF
Global Const $LF = @LF
Global Const $TAB = @TAB
Global Const $DOTLENGTH = StringLen('.')
Global Const $ERROR_NULL_ARRAY[0] = [] ; Empty array constant.
Global Const $HTML_CRLF = '<br />'
Global Const $STR_STRINGSPLIT = BitOR($STR_ENTIRESPLIT, $STR_NOCOUNT)
Global Const $TEMPFILEPATH = _WinAPI_GetTempFileName(@TempDir)

Global Const $PFOP_PATHS = $AUTOITDOCSOURCE & $LF & _
		$AUTOITDOCSOURCE & '\' & $TXTFUNCTIONS & $LF & _
		$AUTOITDOCSOURCE & '\' & $TXTLIBFUNCTIONS & $LF & _
		$AUTOITDOCSOURCE & '\' & $TXTKEYWORDS & $LF & _
		$AUTOITXDOCSOURCE & '\' & $TXTMETHODS & $LF & _
		$AUTOITHTMLSOURCE & $LF
Global Const $PFOPX_PATHS = $AUTOITXDOCSOURCE & $LF & _
		$AUTOITXDOCSOURCE & '\' & $TXTMETHODS & $LF

Global Const $HELP_SECTION_ID = '###'
Global Const $HELP_TABLE_ID = '@@'
#EndRegion Stripped from Help File Check.au3

; Exit error values.
Global Enum $EXIT_ERROR_NONE, _
		$EXIT_ERROR_USERABORT

; Ensure SciTE is not running and then load with the default properties.
Global Const $g_bIsSciTERunningBefore = _SciTE_Run()

#Region Help File Make Constants.
Global Const $HELPGEN_COPYSCRIPT = _HelpGenerate_StripWS('<h2 class="bottom">Example</h2>' & $CRLF & _
		'<script type="text/javascript">' & $CRLF & _
		'if ((navigator.appName == "Microsoft Internet Explorer") && (parseInt(navigator.appVersion) >= 4)) // IE (4+) only' & $CRLF & _
		$TAB & 'function copyToClipboard(section) {' & $CRLF & _
		$TAB & 'if (window.clipboardData && clipboardData.setData) {' & $CRLF & _
		$TAB & $TAB & 'clipboardData.setData("text", section + "\r\n");' & $CRLF & _
		$TAB & $TAB & 'alert("Copied to clipboard");' & $CRLF & _
		$TAB & '}' & $CRLF & _
		'}' & $CRLF & _
		'</script>') & $CRLF ; The EOL is outside _HelpGenerate_StripWS() so it isn't stripped.
Global Const $HELPGEN_EXAMPLE_CODECONTAINER_START = '<div class="codeSnippetContainer">' & $CRLF
Global Const $HELPGEN_EXAMPLE_CODEOPEN_START = _HelpGenerate_StripWS( _ ; Example title.
		$TAB & '<div class="codeSnippetContainerTabs">' & $CRLF & _
		$TAB & $TAB & '<script type="text/javascript">' & $CRLF & _
		$TAB & $TAB & 'if (document.URL.match(/^mk:@MSITStore:/i)) {' & $CRLF & _
		$TAB & $TAB & $TAB & 'document.write(''<div class="codeSnippetContainerTab codeSnippetContainerTabSingle" dir="ltr">'');' & $CRLF & _
		$TAB & $TAB & $TAB & 'document.write(''<object id=hhctrl type="application/x-oleobject" classid="clsid:adb880a6-d8ff-11cf-9377-00aa003b7a11"><param name="Command" value="ShortCut"><param name="Font" value="Verdana,10pt"><param name="Text" value="Text:Open this Script"><param name="Item1" value=",')
Global Const $HELPGEN_EXAMPLE_CODEOPEN_END = _HelpGenerate_StripWS( _ ; Example filename.
		',"></object>'');' & $CRLF & _
		$TAB & $TAB & $TAB & 'document.write(''<\/div>'');' & $CRLF & _
		$TAB & $TAB & '}' & $CRLF & _
		$TAB & $TAB & '</script>' & $CRLF & _
		$TAB & '</div>' & $CRLF) & _
		$CRLF
Global Const $HELPGEN_EXAMPLE_CODECOPY_START = _HelpGenerate_StripWS( _
		'<div class="codeSnippetContainerCodeContainer">' & $CRLF & _
		$TAB & $TAB & '<div class="codeSnippetToolBar">' & $CRLF & _
		$TAB & $TAB & $TAB & '<div class="codeSnippetToolBarText">' & $CRLF & _
		$TAB & $TAB & $TAB & $TAB & '<script type="text/javascript">' & $CRLF & _
		$TAB & $TAB & $TAB & $TAB & 'if ((navigator.appName == "Microsoft Internet Explorer") && (parseInt(navigator.appVersion) >= 4)) // IE (4+) only' & $CRLF & _
		$TAB & $TAB & $TAB & $TAB & $TAB & 'document.write(''<a href="#" id="copy" onclick="copyToClipboard(document.getElementById(\''copytext')
Global Const $HELPGEN_EXAMPLE_CODECOPY_MIDDLE_1 = _HelpGenerate_StripWS( _ ; Element id.
		'\'').innerText)">Copy to clipboard<\/a>'');' & $CRLF & _
		$TAB & $TAB & $TAB & $TAB & '</script>' & $CRLF & _
		$TAB & $TAB & $TAB & '</div>' & $CRLF & _
		$TAB & $TAB & '</div>' & $CRLF & _
		$TAB & $TAB & '<div class="codeSnippetContainerCode" dir="ltr" id="copytext')
Global Const $HELPGEN_EXAMPLE_CODECOPY_MIDDLE_2 = _HelpGenerate_StripWS( _ ; Element id.
		'">' & $CRLF & _
		$TAB & $TAB & $TAB & '<pre>')
Global Const $HELPGEN_EXAMPLE_CODECOPY_END = _HelpGenerate_StripWS( _ ; Example data.
		$CRLF & _
		$TAB & $TAB & $TAB & '</pre>' & $CRLF & _
		$TAB & $TAB & '</div>' & $CRLF & _
		$TAB & '</div>') & $CRLF
Global Const $HELPGEN_EXAMPLE_CODECONTAINER_END = '</div>' & $CRLF ; The EOL is outside _HelpGenerate_StripWS() so it isn't stripped.
Global Const $HELPGEN_EOLCHARS = StringLen($HTML_CRLF & $CRLF)

Global Enum $TABLE_TITLE, _
		$TABLE_DATA, _
		$TABLE_MAX

Global Enum $TABLE_PARSE_FULLDATA, _
		$TABLE_PARSE_TYPE, _
		$TABLE_PARSE_DATA, _
		$TABLE_PARSE_MAX
#EndRegion Help File Make Constants.

OnAutoItExitRegister(_HelpGenerate_Exit)

#cs
	Examples:
	ControlCommand
	DllCall
	_SendMessage
	_WinAPI_EnumResourceNames
#ce

_OutputBuildWrite('TXT to HTML conversion' & @CRLF)

; Check what help file type is being created.
Local $bIsReGenAll = StringInStr($CmdLineRaw, '/RegenAll') > 0
If StringInStr($CmdLineRaw, '/AutoIt') > 0 Then
	GenerateHelpFile($HELP_TYPE_NATIVE, $bIsReGenAll)
	GenerateHelpFile($HELP_TYPE_KEYWORD, $bIsReGenAll)
ElseIf StringInStr($CmdLineRaw, '/UDFs') Then
	GenerateHelpFile($HELP_TYPE_USERDEFINED, $bIsReGenAll)
ElseIf FileExists($sPreviousWorkingDir & '\AutoItX.hhp') Then ; If the AutoItX.hpp file exists in the working directory (before it was reset) then it must be AutoItX.
	GenerateHelpFile($HELP_TYPE_AUTOITX, $bIsReGenAll)
EndIf

; GenerateHelpFile($HELP_TYPE_NATIVE, False)
; GenerateHelpFile($HELP_TYPE_USERDEFINED, False)
; GenerateHelpFile($HELP_TYPE_KEYWORD, False)
; GenerateHelpFile($HELP_TYPE_AUTOITX, False)

#Region Create NATIVE/USERDEFINED/KEYWORD TXT Docs
Func GenerateHelpFile($iType, $bIsReGenAll)
	Local $aExampleFiles = 0, $aFuncs = 0, _
			$bIsMultipleExamples = False, $bIsRenGen = False, _
			$hHelpFile = 0, _
			$iExampleCount = 1, $iHTMLModified = 0, $iSectionType = $HELP_TYPE_NONE, _
			$sKeywordsToRelative = '', $sNativeToRelative = '', $sUserDefinedToRelative = '', _ ; Used in expanding related functions.
			$sExampleDirPath = '', $sExampleFileName = '', $sFuncName = '', $sHelpFileTxtPath = '', $sHTMLDirPath = '', $sHTMLFileName = '', $sHTMLFilePath = '', $sOutputFileDir = '', _
			$sSection = '', $sSectionTemp = '', $sTitleReference = '', $sTitleName = '', _
			$sDescriptionData = '', $sExampleData, $sExperimentalData = '', $sParametersData = '', $sRelatedData = '', $sRemarksData = '', $sReturnData = '', _
			$sSeeAlsoData = '', $sSyntaxData = '', _ ; Sections to append at the end of each file creation.
			$vTemp_1 = Null, $sHttpData, $iTemp

	; Set the filepaths based on the type passed.
	Switch $iType
		Case $HELP_TYPE_AUTOITX
			$sExampleDirPath = $AUTOITXDOCSOURCE & '\' & $EXAMPLES
			$sHelpFileTxtPath = $AUTOITXDOCSOURCE & '\' & $TXTMETHODS
			$sHTMLDirPath = $AUTOITXHTMLSOURCE & '\' & $HTMLMETHODS
			$sOutputFileDir = $HELPGEN_RELATIVE_TXTMETHODS

			$sNativeToRelative = ''
			$sUserDefinedToRelative = ''
			$sKeywordsToRelative = ''

			_OutputBuildWrite('Scanning methods...' & @CRLF)

		Case $HELP_TYPE_KEYWORD
			$sExampleDirPath = $AUTOITDOCSOURCE & '\' & $EXAMPLES
			$sHelpFileTxtPath = $AUTOITDOCSOURCE & '\' & $TXTKEYWORDS
			$sHTMLDirPath = $AUTOITHTMLSOURCE & '\' & $HTMLKEYWORDS
			$sOutputFileDir = $HELPGEN_RELATIVE_TXTKEYWORDS

			$sNativeToRelative = $HELPGEN_RELATIVE_HTMLFUNCTIONS
			$sUserDefinedToRelative = $HELPGEN_RELATIVE_HTMLLIBFUNCTIONS
			$sKeywordsToRelative = ''

			_OutputBuildWrite('Scanning keywords...' & @CRLF)

		Case $HELP_TYPE_NATIVE
			$sExampleDirPath = $AUTOITDOCSOURCE & '\' & $EXAMPLES
			$sHelpFileTxtPath = $AUTOITDOCSOURCE & '\' & $TXTFUNCTIONS
			$sHTMLDirPath = $AUTOITHTMLSOURCE & '\' & $HTMLFUNCTIONS
			$sOutputFileDir = $HELPGEN_RELATIVE_TXTFUNCTIONS

			$sNativeToRelative = ''
			$sUserDefinedToRelative = $HELPGEN_RELATIVE_HTMLLIBFUNCTIONS
			$sKeywordsToRelative = $HELPGEN_RELATIVE_HTMLKEYWORDS

			_OutputBuildWrite('Scanning functions...' & @CRLF)

		Case $HELP_TYPE_USERDEFINED
			$sExampleDirPath = $AUTOITDOCSOURCE & '\' & $LIBEXAMPLES
			$sHelpFileTxtPath = $AUTOITDOCSOURCE & '\' & $TXTLIBFUNCTIONS
			$sHTMLDirPath = $AUTOITHTMLSOURCE & '\' & $HTMLLIBFUNCTIONS
			$sOutputFileDir = $HELPGEN_RELATIVE_TXTLIBFUNCTIONS

			$sNativeToRelative = $HELPGEN_RELATIVE_HTMLFUNCTIONS
			$sUserDefinedToRelative = ''
			$sKeywordsToRelative = $HELPGEN_RELATIVE_HTMLKEYWORDS

			_OutputBuildWrite('Scanning UDFs...' & @CRLF)

	EndSwitch

	; Local constants. So as not to mess up the code with overwriting.
	Local Enum $EXAMPLE_CUSTOM_LINE, $EXAMPLE_CUSTOM_TITLE ; For the custom title array.
	Local Const $sExampleList = _HelpGenerate_GetExampleList($sExampleDirPath) ; Create a list of examples to pass to _HelpGenerate_GetExampleFiles().
	Local $aFilePathTxtList = _FileListToArray($sHelpFileTxtPath, '*.txt', $FLTA_FILES, True)
	If @error Then Return False

	Local Const $STR_LASTCOMMA = StringLen(', '), $STR_LASTSPACE = StringLen(' ')

	; Local $sReferences = ''
	For $i = 1 To UBound($aFilePathTxtList) - 1 ; Loop through each TXT doc.
		; Should check if the help file handle is not Null, but for speed I am going to hope the TXT docs were checked using the help file checker beforehand.
		$hHelpFile = _HelpFileEx($aFilePathTxtList[$i])

		; $sReferences &= $hHelpFile[$HELPFILE_FILENAME] & $ASCII_ACK
		; Set the HTML filename and HTML fullpath variables.
		$sHTMLFileName = $hHelpFile[$HELPFILE_FILENAME] & '.htm'
		$sHTMLFilePath = $sHTMLDirPath & '\' & $sHTMLFileName

		; Retrieve the include filename e.g. FuncName.
		$sExampleFileName = _HelpMisc_GetIncludeFileNameFromData($hHelpFile[$HELPFILE_FILEDATA], $hHelpFile[$HELPFILE_FILENAME])
		$aExampleFiles = _HelpGenerate_GetExampleFiles($sExampleList, $sExampleFileName) ; Examples are case-insensitive, so TCPStartup is similar to TCPStartUp.

		; If set to not re-generate and the txt doc is less than the HTML file, then don't re-generate. Quite simple really!
		If Not $bIsReGenAll Then
			$bIsRenGen = False ; Set to false by default.
			$iHTMLModified = Int(FileGetTime($sHTMLFilePath, $FT_MODIFIED, $FT_STRING))
			If $hHelpFile[$HELPFILE_MODIFIEDDATE] > $iHTMLModified Then
				$bIsRenGen = True ; Set to true .
			Else
				; Check the examples inc. multiple.
				For $sExampleFile In $aExampleFiles
					If Int(FileGetTime($sExampleFile, $FT_MODIFIED, $FT_STRING)) > $iHTMLModified Then ; If the example is greater than the HTML then re-generate.
						$bIsRenGen = True ; Set to true and exit the loop.
						ExitLoop
					EndIf
				Next
			EndIf
			If Not $bIsRenGen Then ; If false move onto the next TXT doc.
				ContinueLoop
			EndIf
		EndIf

		_OutputBuildWrite($sOutputFileDir & $hHelpFile[$HELPFILE_FILENAME] & '.txt' & @CRLF)

		$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to read the next line, which is currently -1 as it's the first time reading the TXT doc.
		$sSection = $hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]] ; Get the section e.g. ###Function###, ###Operator###
		$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to read the next line.
		$sTitleName = $hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]] ; Get the second line.
		$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to read the next line.

		$iSectionType = $HELP_TYPE_NONE
		Switch $sSection
			Case $HELP_SECTION_FUNCTION
				$iSectionType = $HELP_TYPE_NATIVE
			Case $HELP_SECTION_KEYWORD
				$iSectionType = $HELP_TYPE_KEYWORD
			Case $HELP_SECTION_MACRO
				$iSectionType = $HELP_TYPE_MACRO
			Case $HELP_SECTION_METHOD
				$iSectionType = $HELP_TYPE_AUTOITX
			Case $HELP_SECTION_OPERATOR
				$iSectionType = $HELP_TYPE_OPERATOR
			Case $HELP_SECTION_STRUCTURENAME
				$iSectionType = $HELP_TYPE_STRUCTURE
			Case $HELP_SECTION_USERDEFINED
				$iSectionType = $HELP_TYPE_USERDEFINED
		EndSwitch

		$sTitleReference = 'Unspecified'
		Switch $iSectionType
			Case $HELP_TYPE_NATIVE, $HELP_TYPE_USERDEFINED
				$sTitleReference = 'Function'
			Case $HELP_TYPE_KEYWORD, $HELP_TYPE_STRUCTURE
				$sTitleReference = 'Keyword'
			Case $HELP_TYPE_MACRO
				$sTitleReference = 'Macro'
			Case $HELP_TYPE_AUTOITX
				$sTitleReference = 'Method'
			Case $HELP_TYPE_OPERATOR
				$sTitleReference = 'Operator'
		EndSwitch

		; Reset of the section data variables. Maybe an array would be easier or a map?
		$sDescriptionData = ''
		$sExampleData = ''
		$sExperimentalData = ''
		$sFuncName = ''
		$sParametersData = ''
		$sRelatedData = ''
		$sRemarksData = ''
		$sReturnData = ''
		$sSeeAlsoData = ''
		$sSyntaxData = ''
		$vTemp_1 = '' ; This is just a temporary variable when something needs to be stored temporarily.

		; Loop through each line of the TXT doc.
		While $hHelpFile[$HELPFILE_CURRENTLINE] <= $hHelpFile[$HELPFILE_LASTLINE] ; Or we could start from the beginning with $hHelpFile[$HELPGEN_FIRSTLINE].
			$sSectionTemp = '' ; Clear the section temporary data.

			; Process each line of the file.
			Switch $hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]]
				Case $HELP_SECTION_DESCRIPTION
					$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index to skip the section header.
					Do
						If Not _HelpGenerate_IsEmpty($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]]) Then ; Get the line data.
							$sSectionTemp &= $hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]] ; Get the line data.
						EndIf
						$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index.
					Until StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_SECTION_ID) ; If the next line is a section header then exit the loop.
					$hHelpFile[$HELPFILE_CURRENTLINE] -= 1 ; Decrease the current line index to check the next section header.

					$sDescriptionData = '<p class="funcdesc">' & $sSectionTemp & $HTML_CRLF & _
							'</p>' & $CRLF
					; $sReferences &= $sSectionTemp & $ASCII_ACK & @CRLF

				Case $HELP_SECTION_EXAMPLE
					$bIsMultipleExamples = UBound($aExampleFiles) > 1 ; Check if there is more than one example i.e. 2, 3, 4 ...
					$iExampleCount = 1 ; Reset the count to 1.

					$sExampleData = $HELPGEN_COPYSCRIPT ; Add the Javascript code for copying examples.
					For $sExampleFile In $aExampleFiles
						$sSectionTemp = _SciTE_Au3ToHTML($sExampleFile, $TEMPFILEPATH) ; Parse the example to HTML using SciTE.

						#cs
							Custom titles:
							..\english\txt2htm\examples\ConsoleWrite[2].au3
							..\english\txt2htm\examples\StringRegExp.au3
							..\english\txt2htm\examples\StringRegExp[2].au3
							..\english\txt2htm\examples\StringRegExp[3].au3
							..\english\txt2htm\examples\StringRegExp[4].au3
							..\english\txt2htm\examples\TCPConnect[2].au3
							..\english\txt2htm\examples\TCPShutdown.au3
							..\english\txt2htm\examples\TCPShutdown[2].au3
							..\english\txt2htm\examples\TCPStartup.au3
							..\english\txt2htm\examples\TCPStartUp[2].au3

							$vTemp_1 returns:
							[0] ($EXAMPLE_CUSTOM_LINE) = Entire first line.
							[1] ($EXAMPLE_CUSTOM_TITLE) = Title.

							which is based on the output by SciTE e.g. <span><span class="S1">; == Custom Title</span><br />
						#ce
						$vTemp_1 = StringRegExp($sSectionTemp, '(^<span><span\h+class="S\d+">; == (.+)</span>(?:<br\h*/?>)?\R)', $STR_REGEXPARRAYGLOBALMATCH) ; Read the first line of where the title should be. I don't like this assumption  and perhaps the identifier of ; == should be changed.
						If @error Then ; Doesn't appear there is a custom title.
							$vTemp_1 = ($bIsMultipleExamples ? '<h3>Example ' & $iExampleCount & '</h3>' & $CRLF : '') ; Store the title in the temporary variable.
							$sSectionTemp = $CRLF & $sSectionTemp ; Add EOL before the example.
						Else ; There is a custom title.
							$sSectionTemp = StringReplace($sSectionTemp, $vTemp_1[$EXAMPLE_CUSTOM_LINE], '') ; Replace the custom title in the example.
							$vTemp_1 = '<h3>' & $vTemp_1[$EXAMPLE_CUSTOM_TITLE] & '</h3>' ; Store the title in the temporary variable.
						EndIf

						; If the type is AutoItX then the open buttons won't be display. Instead a break is placed there. A workaround at the moment for adding a single space under the title.
						$sSectionTemp = $vTemp_1 & _
								$HELPGEN_EXAMPLE_CODECONTAINER_START & _
								(($iType = $HELP_TYPE_AUTOITX) ? $HTML_CRLF & $CRLF : $HELPGEN_EXAMPLE_CODEOPEN_START & 'Examples\\HelpFile\\' & StringTrimLeft($sExampleFile, StringInStr($sExampleFile, '\', $STR_NOCASESENSEBASIC, -1)) & $HELPGEN_EXAMPLE_CODEOPEN_END) & _ ; Filename.
								$HELPGEN_EXAMPLE_CODECOPY_START & $iExampleCount & $HELPGEN_EXAMPLE_CODECOPY_MIDDLE_1 & $iExampleCount & $HELPGEN_EXAMPLE_CODECOPY_MIDDLE_2 & _
								$sSectionTemp & (($iType = $HELP_TYPE_AUTOITX) ? $HTML_CRLF : '') & _ ; Example data.
								$HELPGEN_EXAMPLE_CODECOPY_END & _
								$HELPGEN_EXAMPLE_CODECONTAINER_END

						$iExampleCount += 1 ; Increase the example count.
						$sExampleData &= $sSectionTemp ; Add to the example output.
					Next

				Case $HELP_SECTION_EXPERIMENTAL
					$sExperimentalData &= '<div class="experimental">Warning: This feature is experimental.  It may not work, may contain bugs or may be changed or removed without notice.' & _
							$HTML_CRLF & _
							$HTML_CRLF & _
							'DO NOT REPORT BUGS OR REQUEST NEW FEATURES FOR THIS FEATURE.' & _
							'</div>' & _
							$HTML_CRLF & $CRLF

				Case $HELP_SECTION_FIELDS, $HELP_SECTION_PARAMETERS ; Parameters for functions and tag constants.
					$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index to skip the section header.

					If Not _HelpGenerate_IsEmpty($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]]) Then ; If not empty i.e. white-space.
						Do
							If StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_TABLE_ID) Then
								If _HelpGenerate_CreateTable($hHelpFile, $vTemp_1) Then
									$sSectionTemp &= $vTemp_1
								EndIf
							Else
								$sSectionTemp &= $hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]] & $HTML_CRLF & $CRLF ; Get the line data.
							EndIf
							$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index.
						Until StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_SECTION_ID) ; If the next line is a section header then exit the loop.
						$hHelpFile[$HELPFILE_CURRENTLINE] -= 1 ; Decrease the current line index to check the next section header.

						If Not _HelpGenerate_IsEmpty($sSectionTemp) Then ; If the temporary data isn't blank add to the section syntax variable.
							$sParametersData = '<h2>Parameters</h2>' & $CRLF & _
									$sSectionTemp & $CRLF & _
									$CRLF
						EndIf
					EndIf

				Case $HELP_SECTION_RELATED
					$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index to skip the section header.

					If Not _HelpGenerate_IsEmpty($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]]) Then ; If the function name isn't blank then add to the see also data variable.
						$aFuncs = StringSplit($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], ', ', $STR_STRINGSPLIT) ; Create an array of related functions or sections based on the assumption that ", " denotes the split.
						For $iFunction = 0 To UBound($aFuncs) - 1
							If Not StringInStr($aFuncs[$iFunction], '<a href=') Then ; If it's a link then skip formatting the hyperlink.
								If StringInStr($aFuncs[$iFunction], '...') Then ; GUI management.
									$aFuncs[$iFunction] = '<a href="' & StringReplace($aFuncs[$iFunction], '...', '%20Management.htm') & '">' & $aFuncs[$iFunction] & '</a>'

								ElseIf StringInStr($aFuncs[$iFunction], '(Option)') Then ; AutoItSetOption.
									$aFuncs[$iFunction] = '<a href="AutoItSetOption.htm#' & StringReplace($aFuncs[$iFunction], '(Option)', '') & '">' & $aFuncs[$iFunction] & '</a>'

								ElseIf StringLeft($aFuncs[$iFunction], $DOTLENGTH) == '.' Then ; Native function inside a UDF function.
									$aFuncs[$iFunction] = StringTrimLeft($aFuncs[$iFunction], $DOTLENGTH) ; Strip the dot.
									$aFuncs[$iFunction] = '<a href="' & (($iSectionType = $HELP_TYPE_NATIVE Or $iSectionType = $HELP_TYPE_AUTOITX) ? '' : $HELPGEN_RELATIVE_HTMLFUNCTIONS) & _
											$aFuncs[$iFunction] & '.txt">' & $aFuncs[$iFunction] & '()</a>' ; In case the dot is present when inside a native function.

								Else ; Assume it's a function/keyword.
									$sFuncName = $aFuncs[$iFunction] ; Store the function/keyword name when creating the hyperlink.
									If $iType = $HELP_TYPE_AUTOITX Then
										$aFuncs[$iFunction] = _WinAPI_PathFindOnPath($aFuncs[$iFunction] & '.txt', $PFOPX_PATHS)
									Else
										$aFuncs[$iFunction] = _WinAPI_PathFindOnPath($aFuncs[$iFunction] & '.txt', $PFOP_PATHS)
									EndIf
									If @error Then
										ContinueLoop ; Continue the loop as there was an error finding the TXT doc.
									Else
										If $iType = $HELP_TYPE_AUTOITX Then ; An improved workaround that could be reduced further.
											$aFuncs[$iFunction] = StringReplace($aFuncs[$iFunction], $AUTOITXDOCSOURCE & '\' & $TXTMETHODS & '\', $sNativeToRelative)
										Else
											$aFuncs[$iFunction] = StringReplace($aFuncs[$iFunction], $AUTOITDOCSOURCE & '\' & $TXTFUNCTIONS & '\', $sNativeToRelative)
											$aFuncs[$iFunction] = StringReplace($aFuncs[$iFunction], $AUTOITDOCSOURCE & '\' & $TXTLIBFUNCTIONS & '\', $sUserDefinedToRelative)
											$aFuncs[$iFunction] = StringReplace($aFuncs[$iFunction], $AUTOITDOCSOURCE & '\' & $TXTKEYWORDS & '\', $sKeywordsToRelative)
										EndIf

										$aFuncs[$iFunction] = '<a href="' & $aFuncs[$iFunction] & '">' & $sFuncName & '</a>'
									EndIf

								EndIf
							EndIf
							$sSectionTemp &= $aFuncs[$iFunction] & ', '
						Next

						$sSectionTemp = StringTrimRight($sSectionTemp, $STR_LASTCOMMA)
						If $sSectionTemp <> '' Then
							$sSectionTemp = StringReplace($sSectionTemp, '.txt', '.htm') ; Convert .txt extensions to .htm.
							$sRelatedData = '<h2>Related</h2>' & $CRLF & _
									'<p>' & $sSectionTemp & '</p>' & $CRLF & _
									$CRLF
						EndIf
					EndIf

				Case $HELP_SECTION_REMARKS
					$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index to skip the section header.

					If Not _HelpGenerate_IsEmpty($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]]) Then ; If not empty i.e. white-space.
						Do
							If StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_TABLE_ID) Then
								If _HelpGenerate_CreateTable($hHelpFile, $vTemp_1) Then
									_HelpGenerate_StripEOL($sSectionTemp)
									$sSectionTemp &= $vTemp_1 & $HTML_CRLF & $CRLF
								EndIf
							Else
								$sSectionTemp &= $hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]] & $HTML_CRLF & $CRLF ; Get the line data.
								_HelpGenerate_ReplaceWS($sSectionTemp)
							EndIf
							$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index.

							; If the next line is a section header or it's greater than the line count then exit the loop.
						Until $hHelpFile[$HELPFILE_CURRENTLINE] > $hHelpFile[$HELPFILE_LASTLINE] Or StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_SECTION_ID)
						$hHelpFile[$HELPFILE_CURRENTLINE] -= 1 ; Decrease the current line index to check the next section header.

						_HelpGenerate_StripEOL($sSectionTemp)

						If Not _HelpGenerate_IsEmpty($sSectionTemp) Then ; If the temporary data isn't blank add to the section remarks variable.
							_HelpGenerate_ReplaceIncludes($sSectionTemp)
							; _HelpGenerate_ConvertTabsToHTML($sSectionTemp)
							$sRemarksData = '<h2>Remarks</h2>' & $CRLF & _
									'<p>' & $sSectionTemp & '</p>' & $CRLF & _
									$CRLF
						EndIf
					EndIf

				Case $HELP_SECTION_RETURNVALUE
					$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index to skip the section header.

					If Not _HelpGenerate_IsEmpty($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], False) Then ; If not empty i.e. white-space.
						Do
							If StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_TABLE_ID) Then
								If _HelpGenerate_CreateTable($hHelpFile, $vTemp_1) Then
									$sSectionTemp &= $vTemp_1 & $HTML_CRLF & $CRLF ; Add an empty line after the table data.
								EndIf
							Else
								$sSectionTemp &= _HelpGenerate_LeadingSpaceToNBSP($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]]) & $HTML_CRLF & $CRLF ; Get the line data.
							EndIf
							$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index.
						Until StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_SECTION_ID) ; If the next line is a section header then exit the loop.
						$hHelpFile[$HELPFILE_CURRENTLINE] -= 1 ; Decrease the current line index to check the next section header.

						_HelpGenerate_StripEOL($sSectionTemp)

						If Not _HelpGenerate_IsEmpty($sSectionTemp) Then ; If the temporary data isn't blank add to the section return variable.
							$sReturnData = '<h2>Return Value</h2>' & $CRLF & _
									$sSectionTemp & $CRLF & _
									$CRLF
						EndIf
					EndIf

				Case $HELP_SECTION_SEEALSO
					$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index to skip the section header.

					If Not _HelpGenerate_IsEmpty($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]]) Then ; If the @@MsdnLink@@ line isn't blank then split at ", " due to some TXT docs having multiple links e.g. _GUICtrlRichEdit_AutoDetectURL().
						$vTemp_1 = StringSplit($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], ', ', $STR_STRINGSPLIT)
						For $iMsdnLink = 0 To UBound($vTemp_1) - 1
							; This can be either be a Win32 API or URL. Generally speaking it will be a Win32 API though there are exceptions.
							$sFuncName = StringRegExpReplace($vTemp_1[$iMsdnLink], '(?:^' & $HELP_TABLE_ID & 'MsdnLink' & $HELP_TABLE_ID & '\h+)', '') ; Get the next line which will be @@MsdnLink@@ FuncName. Retrieve the function name.
							If @extended > 0 Then ; An @@MsdnLink@@ was found.
								If StringLeft($sFuncName, StringLen('gdip')) = 'gdip' Then ; If GDIPlus then continue, else it's a standard Win32 API.
									$sSectionTemp &= 'Search <a href="http://search.msdn.microsoft.com/search/Default.aspx?brand=msdn&query=' & $sFuncName & '">' & _
											$sFuncName & _
											'</a> in MSDN  Library. ' ; Add a single space in case multiple links.
								Else
									$sSectionTemp &= 'Search <a href="http://msdn.microsoft.com/query/dev10.query?appId=Dev10IDEF1&l=EN-US&k=k(' & $sFuncName & ');k(DevLang-C);k(TargetOS-WINDOWS)&rd=true">' & _
											$sFuncName & _
											'</a> in MSDN  Library. ' ; Add a single space in case multiple links.
								EndIf

							Else ; Else it's a URL.
								$iTemp = StringInStr($vTemp_1[$iMsdnLink], ' ')
								If $iTemp Then
									$sHttpData = StringTrimLeft($vTemp_1[$iMsdnLink], $iTemp)
									$vTemp_1[$iMsdnLink] = StringLeft($vTemp_1[$iMsdnLink], $iTemp - 1)
								Else
									$sHttpData = $vTemp_1[$iMsdnLink]
								EndIf
								$sSectionTemp &= 'See <a href="' & $vTemp_1[$iMsdnLink] & '">' & _
										$sHttpData & _
										'</a>. ' ; Add a single space in case multiple links.
							EndIf
						Next
						$sSectionTemp = StringTrimRight($sSectionTemp, $STR_LASTSPACE) & $CRLF ; Strip the last space.
						$sSeeAlsoData = '<h2>See Also</h2>' & $CRLF & _
								'<p>' & $sSectionTemp & '</p>' & $CRLF & _
								$CRLF
					EndIf
					#cs
						These are some exceptions where multiple links are used:
						_GUICtrlRichEdit_AutoDetectURL.txt
						_GUICtrlRichEdit_GetFont.txt
						_GUICtrlRichEdit_SetFont.txt
					#ce

				Case $HELP_SECTION_SYNTAX
					$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index to skip the section header.

					Do
						If Not _HelpGenerate_IsEmpty($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]]) Then ; If not empty i.e. white-space.
							If StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_TABLE_ID) Then
								If _HelpGenerate_CreateTable($hHelpFile, $vTemp_1) Then
									$sSectionTemp &= $vTemp_1
								EndIf
							Else
								$sSectionTemp &= $hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]] & $HTML_CRLF & $CRLF ; Get the line data.
							EndIf
						EndIf
						$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line index.
					Until StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_SECTION_ID) ; If the next line is a section header then exit the loop.
					$hHelpFile[$HELPFILE_CURRENTLINE] -= 1 ; Decrease the current line index to check the next section header.

					If Not _HelpGenerate_IsEmpty($sSectionTemp) Then ; If the temporary data isn't blank add to the section syntax variable.
						_HelpGenerate_ReplaceIncludes($sSectionTemp)
						$sSyntaxData = '<p class="codeheader">' & $CRLF & _
								$sSectionTemp & _
								'</p>' & $CRLF & _
								$CRLF
					EndIf

			EndSwitch
			$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the line count by 1.
		WEnd

		_FileWrite($sHTMLFilePath, '<!DOCTYPE html>' & $CRLF & _
				'<html>' & $CRLF & _
				'<head>' & $CRLF & _
				'<title>' & $sTitleReference & ' ' & $sTitleName & '</title>' & $CRLF & _
				'<meta charset="utf-8">' & $CRLF & _
				'<link href="../' & (($iSectionType = $HELP_TYPE_AUTOITX) ? '../' : '') & 'css/default.css" rel="stylesheet">' & $CRLF & _
				'</head>' & $CRLF & _
				$CRLF & _
				'<body>' & $CRLF & _
				$sExperimentalData & _ ; Experimental section.
				'<h1 class="small">' & $sTitleReference & ' Reference</h1>' & $CRLF & _ ; Function/keyword header.
				'<hr style="height:0px">' & $CRLF & _
				'<h1>' & $sTitleName & '</h1>' & $CRLF & _
				$sDescriptionData & _ ; Description section. [NOTE: This should be on line 13 for the management generation to work !!!].
				$sSyntaxData & _ ; Syntax section.
				$sParametersData & _ ; Params section.
				$sReturnData & _ ; Return section.
				$sRemarksData & _ ; Remarks section.
				$sRelatedData & _ ; Related section.
				$sSeeAlsoData & _ ; MSDN section.
				$sExampleData & _ ; Example section.
				'</body>' & $CRLF & _
				'</html>' & $CRLF, $FO_UTF8)
	Next
	; ConsoleWrite($sReferences & @CRLF)
	Return True
EndFunc   ;==>GenerateHelpFile

#Region Stripped from HelpFileLib.au3
Func _HelpFileEx($sFilePath) ; Create a help file "object". This is a modified function of _HelpFile() from HelpFileLib.au3. As speed is essential.
	Local $sData = FileRead($sFilePath)
	; If @error Then Return Null ; File doesn't exist. ; Not required.

	Local Const $STR_STRINGSPLIT = BitOR($STR_ENTIRESPLIT, $STR_NOCOUNT)
	Local $aHelpFile = StringSplit($HELPFILE_DETAILS & $sData, @CRLF, $STR_STRINGSPLIT)
	$aHelpFile[$HELPFILE_UBOUND] = UBound($aHelpFile)
	; __HelpFile_SetExamplePath($aHelpFile, $sExamplePath)
	$aHelpFile[$HELPFILE_FILEPATH] = $sFilePath

	$aHelpFile[$HELPFILE_FILENAME] = StringInStr($aHelpFile[$HELPFILE_FILEPATH], '\', $STR_NOCASESENSEBASIC, -1) ; Should be a different variable to store the position of \, but lets double up for speed.
	$aHelpFile[$HELPFILE_FILEDIR] = StringLeft($aHelpFile[$HELPFILE_FILEPATH], $aHelpFile[$HELPFILE_FILENAME] - 1) ; Trim the filespec.

	$aHelpFile[$HELPFILE_FILENAME] = StringTrimLeft($aHelpFile[$HELPFILE_FILEPATH], $aHelpFile[$HELPFILE_FILENAME]) ; Trim the filepath by using the position of \.
	$aHelpFile[$HELPFILE_FILENAME] = StringLeft($aHelpFile[$HELPFILE_FILENAME], StringInStr($aHelpFile[$HELPFILE_FILENAME], '.', $STR_NOCASESENSEBASIC, -1) - 1) ; Trim the extension.

	Local Const $FT_STRING = 1
	$aHelpFile[$HELPFILE_MODIFIEDDATE] = Int(FileGetTime($aHelpFile[$HELPFILE_FILEPATH], $FT_MODIFIED, $FT_STRING))
	; If @error THen $aHelpFile[$HELPFILE_MODIFIEDDATE] = Null ; Not required.

	$aHelpFile[$HELPFILE_FIRSTLINE] = $HELPFILE_MAX
	$aHelpFile[$HELPFILE_LASTLINE] = $aHelpFile[$HELPFILE_UBOUND] - 1
	$aHelpFile[$HELPFILE_LINECOUNT] = $aHelpFile[$HELPFILE_UBOUND] - $HELPFILE_MAX

	; This can be changed only! It's the current line in the internal object.
	$aHelpFile[$HELPFILE_CURRENTLINE] = $aHelpFile[$HELPFILE_FIRSTLINE] - 1 ; Minus 1, so when the likes of _HelpFile_GetNextLine() are first called it retrieves the first line.

	$aHelpFile[$HELPFILE_FILEDATA] = $sData ; Only for comparison during the re-building stage.
	; $aHelpFile[$HELPFILE_BUILDDATA] = $aHelpFile[$HELPFILE_FILEDATA] ; Correctly formatted version of the text file.

	; _ConvertEOLToCRLF($aHelpFile[$HELPFILE_BUILDDATA])
	; _StripTrailingWhitespace($aHelpFile[$HELPFILE_BUILDDATA])
	; _StripEmptyLinesDouble($aHelpFile[$HELPFILE_BUILDDATA])

	; $aHelpFile[$HELPFILE_ISTAG] = StringRegExp($aHelpFile[$HELPFILE_FILENAME], '^\$tag') > 0
	; $aHelpFile[$HELPFILE_ISTAG] = StringRegExp($aHelpFile[$HELPFILE_BUILDDATA], '^\h*###Structure Name###\R') > 0
	; $aHelpFile[$HELPFILE_TYPE] = $HELPFILE_TYPE_NONE

	#cs
		$aHelpFile[$HELPFILE_SECTIONS] = 0
		If __HelpFileEx_GenerateSections($aHelpFile) Then
		$aHelpFile[$HELPFILE_ID] = $HELPFILE_GUID ; Set the GUID so as to check later on that in fact it is a valid help file handle.
		_HelpFileEx_BuildSectionData($aHelpFile)
		EndIf
	#ce

	Return $aHelpFile
EndFunc   ;==>_HelpFileEx
#EndRegion Stripped from HelpFileLib.au3

Func _HelpGenerate_CreateTable(ByRef $hHelpFile, ByRef $sTableData)
	; Reset the out parameter.
	$sTableData = ''

	Local $bReturn = True
	Switch $hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]]
		Case $HELP_TABLE_ID & 'ParamTable' & $HELP_TABLE_ID
			$sTableData = _HelpGenerate_ParamTable($hHelpFile)
		Case $HELP_TABLE_ID & 'ReturnTable' & $HELP_TABLE_ID
			$sTableData = _HelpGenerate_ReturnTable($hHelpFile)
		Case $HELP_TABLE_ID & 'ControlCommandTable' & $HELP_TABLE_ID
			$sTableData = _HelpGenerate_ControlCommandTable($hHelpFile)
		Case $HELP_TABLE_ID & 'StandardTable' & $HELP_TABLE_ID
			$sTableData = _HelpGenerate_StandardTable($hHelpFile)
		Case $HELP_TABLE_ID & 'StandardTable1' & $HELP_TABLE_ID
			$sTableData = _HelpGenerate_StandardTable1($hHelpFile)
		Case Else
			$bReturn = False
	EndSwitch
	Return $bReturn
EndFunc   ;==>_HelpGenerate_CreateTable

Func _HelpGenerate_Exit()
	If $g_bIsSciTERunningBefore Then
		; Close the open files.
		_SciTE_CloseAll()
		; Reload the saved session.
		_SciTE_LoadSession(_SciTE_GetSessionPath())
		; Reload the SciTE properties.
		_SciTE_ReloadProperties()
	Else
		_SciTE_Quit()
	EndIf

	; Close the SciTE processing functions.
	_SciTE_Shutdown()

	; Delete the temporary filepaths.
	FileDelete($TEMPFILEPATH)

	Exit $EXIT_ERROR_NONE
EndFunc   ;==>_HelpGenerate_Exit

; Possible feature: @@IncludeExample@@ Example_1, Example_2, Example[4]
Func _HelpGenerate_GetExampleFiles(ByRef Const $sExampleList, $sFileName)
	Local $aExamples = $ERROR_NULL_ARRAY
	If $sExampleList <> '' Then
		; $sFileName = StringLeft($sFileName, StringInStr($sFileName, '.', $STR_NOCASESENSEBASIC, -1) - 1) ; Trim the extension.
		$sFileName = StringRegExpReplace($sFileName, '(?:\.\w{3,}$)', '') ; Trim the extension. (guinness' way).
		$aExamples = StringRegExp($sExampleList, '(?i)((?<=\n).*\\' & $sFileName & '(?:\[\d+\])?\.\w{3,}(?=\n))', $STR_REGEXPARRAYGLOBALMATCH)
		If @error Then
			$aExamples = $ERROR_NULL_ARRAY
		EndIf
	EndIf
	Return $aExamples
EndFunc   ;==>_HelpGenerate_GetExampleFiles

; Possible feature: Multiple example paths. This is a new feature that can be added later on in which EXAMPLEPATH_1|EXAMPLEPATH_2 would allow for native functions in UDFs and vice versa. Not currently implemented yet.
Func _HelpGenerate_GetExampleList($sFilePath)
	Local $aFileList = StringSplit($sFilePath, '|', $STR_STRINGSPLIT), _
			$aFilePaths = 0, $sExampleList = ''
	For $sFileList In $aFileList
		$aFilePaths = _FileListToArray($sFileList, '*', $FLTA_FILES, True) ; All files in the examples directory.
		If Not @error Then
			$sExampleList &= $LF
			For $i = 1 To UBound($aFilePaths) - 1
				$sExampleList &= $aFilePaths[$i] & $LF
			Next
		EndIf
	Next
	Return $sExampleList
EndFunc   ;==>_HelpGenerate_GetExampleList

Func _HelpGenerate_IsEmpty(ByRef $sData, $bCheckNone = True)
	Return $sData = '' Or StringIsSpace($sData) Or ($bCheckNone ? StringRegExp($sData, '(?i:^\h*\bNone\b\.?\h*$)') : 0)
EndFunc   ;==>_HelpGenerate_IsEmpty

Func _HelpGenerate_ReplaceIncludes(ByRef $sData)
	Local $bReturn = False
	If StringInStr($sData, '#include <') Then ; Escape < > in the include line.
		Local $aArray = StringRegExp($sData, '(?i:)(#include\h*["''<].*?["''>])', $STR_REGEXPARRAYMATCH)
		If Not @error Then
			Local $iExtended = 0, _
					$sIncludeEscaped = ''
			For $i = 0 To UBound($aArray) - 1
				$sIncludeEscaped = $aArray[$i]
				_ConvertSymbolsToHTMLEntity($sIncludeEscaped)
				$sData = StringReplace($sData, $aArray[$i], $sIncludeEscaped, $STR_REPLACE_ONCE)
				$iExtended += @extended
			Next
			$bReturn = $iExtended > 0
		EndIf
	EndIf
	Return $bReturn
EndFunc   ;==>_HelpGenerate_ReplaceIncludes

Func _HelpGenerate_ReplaceWS(ByRef $sData, $sChar = $TAB)
	$sData = StringReplace($sData, $sChar, '&nbsp;&nbsp;&nbsp; ') ; Replace tabs.
	Return True
EndFunc   ;==>_HelpGenerate_ReplaceWS

Func _HelpGenerate_StripEOL(ByRef $sData)
	$sData = StringTrimRight($sData, $HELPGEN_EOLCHARS) ; Strip last EOL characters.
	Return True
EndFunc   ;==>_HelpGenerate_StripEOL

Func _HelpGenerate_StripWS($sData) ; Only used at the start to remove white-space for the constants. Which are retained for readability purposes.
	_StripWhitespace($sData)
	Return $sData
EndFunc   ;==>_HelpGenerate_StripWS
#EndRegion Create NATIVE/USERDEFINED/KEYWORD TXT Docs

#cs
	@@ParamTable@@
	Gtr than 1:
	..\english\txt2htm\txtfunctions\AutoItSetOption.txt
	..\english\txt2htm\txtfunctions\BitRotate.txt
	..\english\txt2htm\txtfunctions\BlockInput.txt
	..\english\txt2htm\txtfunctions\DllCall.txt
	..\english\txt2htm\txtfunctions\DllCallAddress.txt
	..\english\txt2htm\txtfunctions\DllStructCreate.txt
	..\english\txt2htm\txtfunctions\DriveGetFileSystem.txt
	..\english\txt2htm\txtfunctions\DriveStatus.txt
	..\english\txt2htm\txtfunctions\HotKeySet.txt
	..\english\txt2htm\txtfunctions\MouseGetPos.txt
	..\english\txt2htm\txtfunctions\MsgBox.txt
	..\english\txt2htm\txtfunctions\ObjCreateInterface.txt
	..\english\txt2htm\txtfunctions\OnAutoItExitRegister.txt
	..\english\txt2htm\txtfunctions\StringRegExp.txt
	..\english\txt2htm\txtlibfunctions\_IEAction.txt
	..\english\txt2htm\txtlibfunctions\_IEPropertyGet.txt
	..\english\txt2htm\txtlibfunctions\_IEPropertySet.txt
	..\english\txt2htm\txtKeywords\include.txt

	@@ReturnTable@@
	Gtr than 1:
	..\english\txt2htm\txtfunctions\StringRegExp.txt
	..\english\txt2htm\txtlibfunctions\_viExecCommand.txt

	@@ControlCommandTable@@:
	..\english\txt2htm\txtfunctions\ControlCommand.txt
	..\english\txt2htm\txtfunctions\ControlListView.txt
	..\english\txt2htm\txtfunctions\ControlTreeView.txt

	@@StandardTable@@
	..\english\txt2htm\txtfunctions\ObjEvent.txt

	@@StandardTable1@@
	..\english\txt2htm\txtfunctions\ControlClick.txt
	..\english\txt2htm\txtfunctions\ControlTreeView.txt
	..\english\txt2htm\txtfunctions\GUICreate.txt
	..\english\txt2htm\txtfunctions\GUICtrlRead.txt
	..\english\txt2htm\txtfunctions\GUICtrlRegisterListViewSort.txt
	..\english\txt2htm\txtfunctions\GUICtrlSetGraphic.txt
	..\english\txt2htm\txtfunctions\GUICtrlSetResizing.txt
	..\english\txt2htm\txtfunctions\GUICtrlSetState.txt
	..\english\txt2htm\txtfunctions\GUIRegisterMsg.txt
	..\english\txt2htm\txtfunctions\GUISetOnEvent.txt
	..\english\txt2htm\txtfunctions\MouseClick.txt
	..\english\txt2htm\txtfunctions\MsgBox.txt
	..\english\txt2htm\txtfunctions\Send.txt
	..\english\txt2htm\txtfunctions\StringFormat.txt
	..\english\txt2htm\txtfunctions\TrayItemSetState.txt
	..\english\txt2htm\txtfunctions\TraySetOnEvent.txt
#ce
#Region Stripped from the old library and modified to work efficiently for now.
Func _HelpGenerate_ControlCommandTable(ByRef $hHelpFile)
	$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to skip the table start.

	; Make a pass to determine how many rows and columns the table has.
	Local $iIndention = 0, $iLineIndex = $hHelpFile[$HELPFILE_CURRENTLINE], $iRows = 0, $iStart = $hHelpFile[$HELPFILE_CURRENTLINE]
	While Not StringInStr($hHelpFile[$iLineIndex], $HELP_TABLE_ID & 'End' & $HELP_TABLE_ID)
		$iIndention = _HelpGenerate_StringCount($hHelpFile[$iLineIndex], $TAB)
		If Not $iIndention Then $iRows += 1
		$iLineIndex += 1 ; Increase the current line count to read the next line.
	WEnd
	$hHelpFile[$HELPFILE_CURRENTLINE] = $iLineIndex + 1 ; Increase the current line count to skip the table end.
	Local $iStop = $iLineIndex - 1 ; So as not to include the table end.

	Local $aMatrix[$iRows][2], _
			$iColumn = 0, $iRow = -1, _
			$sData = ''
	For $i = $iStart To $iStop
		$iIndention = _HelpGenerate_StringCount($hHelpFile[$i], $TAB)
		$sData = _HelpGenerate_SpaceToNBSP(StringTrimLeft($hHelpFile[$i], $iIndention))
		If Not $iIndention Then
			$iRow += 1
			$iColumn = 0
		Else
			$iColumn = 1
		EndIf
		If $aMatrix[$iRow][$iColumn] <> '' Then
			$aMatrix[$iRow][$iColumn] &= $HTML_CRLF & $sData
		Else
			$aMatrix[$iRow][$iColumn] = $sData
		EndIf
	Next

	Local $sTableData = ''
	For $i = 0 To $iRows - 1
		$sTableData &= '<tr>' & $CRLF
		For $j = 0 To 1
			$sData = StringReplace($aMatrix[$i][$j], $HTML_CRLF, $HTML_CRLF & $CRLF)
			Select
				Case $i = 0 And $j = 0 ; Only put width%-info on first rows
					$sTableData &= '<td style="width:40%">' & $sData & '</td>' & $CRLF
				Case $i = 0 And $j = 1
					$sTableData &= '<td style="width:60%">' & $sData & '</td>' & $CRLF
				Case Else
					$sTableData &= '<td>' & $sData & '</td>' & $CRLF
			EndSelect
		Next
		$sTableData &= '</tr>' & $CRLF
	Next

	If Not (StringStripWS($sTableData, $STR_STRIPALL) == '') Then _
			$sTableData = _
			'<table>' & $CRLF & _
			$sTableData & _
			'</table>'
	Return $sTableData
EndFunc   ;==>_HelpGenerate_ControlCommandTable

Func _HelpGenerate_ParamTable(ByRef $hHelpFile)
	$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to skip the table start.

	; Make a pass to determine how many rows and columns the table has.
	Local $iIndention = 0, $iLineIndex = $hHelpFile[$HELPFILE_CURRENTLINE], $iRows = 0, $iStart = $hHelpFile[$HELPFILE_CURRENTLINE]
	While Not StringInStr($hHelpFile[$iLineIndex], $HELP_TABLE_ID & 'End' & $HELP_TABLE_ID)
		$iIndention = _HelpGenerate_StringCount($hHelpFile[$iLineIndex], $TAB)
		If Not $iIndention And $hHelpFile[$iLineIndex] <> '' Then $iRows += 1 ; If no indentation and the line isn't empty, it's a new row.
		$iLineIndex += 1 ; Increase the current line count to read the next line.
	WEnd
	$hHelpFile[$HELPFILE_CURRENTLINE] = $iLineIndex + 1 ; Increase the current line count to skip the table end.
	Local $iStop = $iLineIndex - 1 ; So as not to include the table end.

	Local $aMatrix[$iRows][2], _
			$iColumn = 0, $iRow = -1, _
			$sData = ''
	For $i = $iStart To $iStop
		$iIndention = _HelpGenerate_StringCount($hHelpFile[$i], $TAB)
		If Not $iIndention And $hHelpFile[$i] <> '' Then ; If no indentation and the line isn't empty, it's a new row.
			$iRow += 1
			$iColumn = 0
		Else
			$iIndention = 1 ; just remove the first $TAB
			$iColumn = 1
		EndIf
		$sData = _HelpGenerate_SpaceToNBSP(StringTrimLeft($hHelpFile[$i], $iIndention))

		$sData = StringReplace($sData, $OPTIONAL, '<strong>' & $OPTIONAL & '</strong>') ; Replace [optional].
		_HelpGenerate_ReplaceWS($sData, $TAB & $TAB) ; This is a workaround for now.

		If $aMatrix[$iRow][$iColumn] <> '' Then
			$aMatrix[$iRow][$iColumn] &= $HTML_CRLF & $sData
		Else
			$aMatrix[$iRow][$iColumn] = $sData
		EndIf
	Next

	Local $sTableData = ''
	For $i = 0 To $iRows - 1
		$sTableData &= '<tr>' & $CRLF
		For $j = 0 To 1
			$sData = StringReplace($aMatrix[$i][$j], $HTML_CRLF, $HTML_CRLF & $CRLF)
			Select
				Case $i = 0 And $j = 0 ; Only put width%-info on first row
					$sTableData &= '<td style="width:15%">' & $sData & '</td>' & $CRLF
				Case $i = 0 And $j = 1
					$sTableData &= '<td style="width:85%">' & $sData & '</td>' & $CRLF
				Case Else
					$sTableData &= '<td>' & $sData & '</td>' & $CRLF
			EndSelect
		Next
		$sTableData &= '</tr>' & $CRLF
	Next

	If Not (StringStripWS($sTableData, $STR_STRIPALL) == '') Then _
			$sTableData = _
			'<table>' & $CRLF & _
			$sTableData & _
			'</table>'
	Return $sTableData
EndFunc   ;==>_HelpGenerate_ParamTable

Func _HelpGenerate_ReturnTable(ByRef $hHelpFile)
	; Assumes one line per row with tab-separated columns.

	$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to skip the table start.
	Local $aTabSplit = StringSplit($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $TAB, $STR_STRINGSPLIT)
	If @error Then
		Local $sTableRawData = ""
		While Not StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_TABLE_ID & 'End' & $HELP_TABLE_ID)
			$sTableRawData &= $HTML_CRLF & $CRLF & _HelpGenerate_LeadingSpaceToNBSP($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]])
			$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to read the next line.
		WEnd
		$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to skip the table end.

		Return $sTableRawData
	EndIf
	Local $sTableData = $CRLF & '<tr>' & $CRLF ; Will contain the final html table code.
	If Not StringInStr($aTabSplit[$TABLE_TITLE], ':') Then
		$aTabSplit[$TABLE_TITLE] &= ':'
	EndIf

	$sTableData &= '<td style="width:10%" class="valign-top">' & $aTabSplit[$TABLE_TITLE] & '</td>' & $CRLF
	$sTableData &= '<td style="width:90%">' & $aTabSplit[$TABLE_DATA]
	$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to read the next line.

	While Not StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_TABLE_ID & 'End' & $HELP_TABLE_ID)
		; Split each row into an array for easy parsing.
		$aTabSplit = StringSplit($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $TAB, $STR_STRINGSPLIT)
		If Not @error And UBound($aTabSplit) >= $TABLE_MAX Then
			If $aTabSplit[$TABLE_TITLE] = '' Then
				$sTableData &= $HTML_CRLF & $CRLF & _HelpGenerate_LeadingSpaceToNBSP(StringTrimLeft($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], 1)) ; 1 = StringLen($TAB)
			Else
				$sTableData &= '</td>' & $CRLF & _
						'</tr>' & $CRLF & _
						'<tr>' & $CRLF
				If Not StringInStr($aTabSplit[$TABLE_TITLE], ':') Then $aTabSplit[$TABLE_TITLE] &= ':' ; Force : if missing.
				$sTableData &= '<td class="valign-top">' & $aTabSplit[$TABLE_TITLE] & '</td>' & $CRLF
				$sTableData &= '<td>' & $aTabSplit[$TABLE_DATA]
			EndIf
		EndIf
		$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to read the next line.
	WEnd
	$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to skip the table end.

	If Not (StringStripWS($sTableData, $STR_STRIPALL) == '') Then _
			$sTableData = _
			'<table class="noborder">' & _
			$sTableData & '</td>' & $CRLF & _
			'</tr>' & $CRLF & _
			'</table>'
	Return $sTableData
EndFunc   ;==>_HelpGenerate_ReturnTable

Func _HelpGenerate_StandardTable(ByRef $hHelpFile)
	; Assumes one line per row with tab-separated columns.
	Local $aTabSplit = 0, _
			$sTableData = '' ; Will contain the final html table code.

	$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to skip the table start.
	While Not StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_TABLE_ID & 'End' & $HELP_TABLE_ID)
		; Split each row into an array for easy parsing.
		$aTabSplit = StringSplit($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $TAB, $STR_STRINGSPLIT)
		If Not @error Then
			$sTableData &= $CRLF & '<tr>' & $CRLF
			For $i = 0 To UBound($aTabSplit) - 1
				$sTableData &= '<td>' & $aTabSplit[$i] & '</td>' & $CRLF
			Next
			$sTableData &= '</tr>' & $CRLF
		EndIf
		$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to read the next line.
	WEnd
	$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to skip the table end.

	If Not (StringStripWS($sTableData, $STR_STRIPALL) == '') Then _
			$sTableData = _
			'<table>' & _
			$sTableData & _
			'</table>'
	Return $sTableData
EndFunc   ;==>_HelpGenerate_StandardTable

Func _HelpGenerate_StandardTable1(ByRef $hHelpFile)
	; Assumes one line per row with tab-separated columns.
	Local $aTabSplit = 0, _
			$sTD = 'th', $sTR = '<tr>', $sTableData = '' ; Will contain the final html table code.

	$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to skip the table start.
	While Not StringInStr($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $HELP_TABLE_ID & 'End' & $HELP_TABLE_ID)
		; Split each row into an array for easy parsing.
		$aTabSplit = StringSplit($hHelpFile[$hHelpFile[$HELPFILE_CURRENTLINE]], $TAB, $STR_STRINGSPLIT)
		If Not @error Then
			$sTableData &= $CRLF & $sTR & $CRLF
			For $i = 0 To UBound($aTabSplit) - 1
				$sTableData &= '<' & $sTD & '>' & $aTabSplit[$i] & '</' & $sTD & '>' & $CRLF
			Next
			$sTableData &= '</tr>' & $CRLF

			; Reset separation line for next lines
			$sTR = '<tr>'
			$sTD = 'td'
		EndIf
		$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to read the next line.
	WEnd
	$hHelpFile[$HELPFILE_CURRENTLINE] += 1 ; Increase the current line count to skip the table end.

	If Not (StringStripWS($sTableData, $STR_STRIPALL) == '') Then _
			$sTableData = $HTML_CRLF & $CRLF & _
			'<table>' & _
			$sTableData & _
			'</table>'
	Return $sTableData
EndFunc   ;==>_HelpGenerate_StandardTable1

Func _HelpGenerate_SpaceToNBSP($sString)
	Local $iOccurrence = 0
	For $i = 1 To StringLen($sString)
		If StringMid($sString, $i, 1) <> ' ' Then
			ExitLoop
		EndIf
		$iOccurrence += 1
	Next
	If $iOccurrence > 0 Then $sString = StringReplace($sString, ' ', '&nbsp;', $iOccurrence)

	$iOccurrence = 0 ; Reset to zero.

	; Also convert each leading tab to 4 spaces ('&nbsp;&nbsp;&nbsp; ')
	For $i = 1 To StringLen($sString)
		If StringMid($sString, $i, 1) <> $TAB Then
			ExitLoop
		EndIf
		$iOccurrence += 1
	Next
	If $iOccurrence > 0 Then $sString = StringReplace($sString, $TAB, '&nbsp;&nbsp;&nbsp; ', $iOccurrence)

	Return $sString
EndFunc   ;==>_HelpGenerate_SpaceToNBSP

Func _HelpGenerate_LeadingSpaceToNBSP($sString)
	Local $iOccurrence = 0
	; Convert each leading tab to 4 spaces ('&nbsp;&nbsp;&nbsp; ')
	For $i = 1 To StringLen($sString)
		If StringMid($sString, $i, 1) = $TAB Then
			$iOccurrence += 1
			If StringMid($sString, $i + 1, 1) = $TAB Then ContinueLoop
			ExitLoop
		Else
			ExitLoop
		EndIf
	Next
	If $iOccurrence > 0 Then $sString = StringReplace($sString, $TAB, '&nbsp;&nbsp;&nbsp; ', $iOccurrence)

	; Also convert next leading " " to ('&nbsp;')
	Local $iFirst = $iOccurrence * 19 + 1

	For $i = $iFirst To StringLen($sString)
		If StringMid($sString, $i, 1) = ' ' Then
			$iOccurrence += 1
			If StringMid($sString, $i + 1, 1) = ' ' Then ContinueLoop
			ExitLoop
		EndIf
	Next

	If $iOccurrence > 0 Then $sString = StringReplace($sString, ' ', '&nbsp;', $iOccurrence)

	Return $sString
EndFunc   ;==>_HelpGenerate_LeadingSpaceToNBSP

Func _HelpGenerate_StringCount($sString, $sChar)
	Local $aArray = StringSplit($sString, $sChar)
	Return (@error ? 0 : $aArray[0] - 1)
EndFunc   ;==>_HelpGenerate_StringCount
#EndRegion Stripped from the old library and modified to work efficiently for now.

Func _HelpGenerate_OnQuit()
	_OutputWaitClosed() ; ### When run in standalone
EndFunc   ;==>_HelpGenerate_OnQuit
